// Generated by CoffeeScript 2.4.1
(function() {
  // Usage:

  // - Existing message with the given ID:

  // ```
  // new Message(ctx, User, id)
  // ```

  // - New message:

  // ```
  // new Message(ctx, User).create()
  // ```
  var Formats, Message, Messaging, assert, debug, foot, heal, pkg, sleep, sum_of, timestamp;

  assert = require('assert');

  sum_of = function(a) {
    var k, v;
    return ((function() {
      var results;
      results = [];
      for (k in a) {
        v = a[k];
        results.push(v);
      }
      return results;
    })()).reduce((function(x, y) {
      return x + y;
    }), 0);
  };

  assert.equal(0, sum_of({}));

  assert.equal(0, sum_of({
    a: 0
  }));

  assert.equal(0, sum_of({
    a: -3,
    b: 3
  }));

  assert.equal(10, sum_of({
    a: 4,
    b: 3,
    c: 2,
    d: 1
  }));

  Formats = require('./Formats');

  Message = (function() {
    var ref, ref1, ref2;

    class Message {
      constructor(ctx, user1, id) {
        this.ctx = ctx;
        this.user = user1;
        this.id = id;
        this.part = this.the_first_part;
        this.done = false;
      }

      uri(name, rev) {
        return this.ctx.voicemail_uri(this.user, this.id, name, rev);
      }

      async get_part(part = this.part) {
        var doc;
        doc = (await this.user.db.get(this.id));
        return Formats.find(doc, `part${part}`);
      }

      // Record the current part
      // -----------------------
      async start_recording() {
        var doc, name, record_seconds, upload_url;
        debug('start_recording', this.id);
        record_seconds = null;
        doc = (await this.user.db.get(this.id));
        if (doc.durations == null) {
          doc.durations = {};
        }
        await this.ctx.set({
          RECORD_TITLE: `Call from ${doc.caller_id}`
        });
        name = Formats.name(`part${this.part}`);
        upload_url = this.uri(name, doc._rev);
        record_seconds = parseInt((await this.ctx.prompt.record(upload_url, this.max_duration)), 10);
        debug('start_recording: message saved', {record_seconds});
        if ((isNaN(record_seconds)) || record_seconds < this.min_duration) {
          await this.delete_single_part(this.part);
          return 0;
        } else {
          doc = (await this.user.db.get(this.id));
          if (doc.durations == null) {
            doc.durations = {};
          }
          doc.durations[name] = record_seconds;
          doc.duration = sum_of(doc.durations);
          await this.user.db.put(doc);
          return record_seconds;
        }
      }

      // Play a recording, optionally collect a digit
      // ------------------------------------------------------------
      async play_recording(this_part = this.the_first_part) {
        var choice, name, url;
        debug('play_recording', this.id, this_part);
        if (!(this_part <= this.the_last_part)) {
          return;
        }
        // Might need to add parameters (`url_params`, between `()`) here; names are:
        // - ext
        // - nohead (skip querying with HEAD; this is used to cache files)
        // See `file_open` in mod_httapi.c.
        name = (await this.get_part(this_part));
        if (name) {
          url = this.uri(name);
          debug('play_recording', {name, url});
          choice = (await this.ctx.prompt.play(url));
        }
        // Keep playing if no user interaction
        if (choice == null) {
          return this.play_recording(this_part + 1);
        } else {
          return choice;
        }
      }

      // Delete parts
      // ------------
      async delete_all_parts() {
        var doc;
        debug('delete_all_parts', this.id);
        doc = (await this.user.db.get(this.id));
        // Remove all attachments
        doc._attachments = {};
        doc.durations = {};
        doc.duration = 0;
        return (await this.user.db.put(doc));
      }

      async delete_single_part(this_part) {
        var doc, name, rev;
        debug('delete_single_part', this_part);
        doc = (await this.user.db.get(this.id));
        if (doc.durations == null) {
          doc.durations = {};
        }
        while (name = Formats.find(doc, `part${this_part}`)) {
          if (name in doc.durations) {
            delete doc.durations[name];
          }
          doc.duration = sum_of(doc.durations);
          ({rev} = (await this.user.db.put(doc)));
          await this.user.db.deleteAttachment(this.id, name, rev).catch(function(error) {
            return debug(`remove attachment: ${error}`, {id: this.id, name, rev});
          });
        }
      }

      // Post-recording menu
      // -------------------
      async post_recording() {
        var choice, it_does;
        debug('post_recording', this.id);
        if (this.done) {
          return;
        }
        // Check whether the attachment exists (it might be deleted if it doesn't match the minimum duration)
        it_does = (await this.get_part(this.part));
        debug('post_recording', {it_does});
        if (!it_does) {
          await debug.ops("Could not record message part", {
            message_id: this.id,
            user_id: this.user.id
          });
          await this.ctx.action('phrase', "could not record please try again");
          await this.start_recording();
          return;
        }
        // FIXME The default FreeSwitch prompts only allow for one-part messages, while we allow for multiple.
        choice = (await this.ctx.prompt.get_choice('phrase:voicemail_record_file_check:1:2:3'));
        switch (choice) {
          // Play
          case "1":
            debug('post_recording: play');
            await this.play_recording(this.the_first_part);
            return this.post_recording();
          // Delete
          case "3":
            debug('post_recording: delete');
            await this.delete_all_parts();
            this.part = this.the_first_part;
            await this.start_recording();
            return this.post_recording();
          // Append
          case "4": // Keep recording
            debug('post_recording: append');
            if (this.part < this.the_last_part) {
              this.part++;
              return (await this.start_recording());
            } else {
              // FIXME: notify that the last part has been recorded
              return this.post_recording();
            }
            break;
          // Save
          case "2":
            debug('post_recording: save');
            break;
          default:
            return this.post_recording();
        }
      }

      // Play the message enveloppe
      // --------------------------
      async play_enveloppe(index) {
        var doc, user_timestamp;
        debug('play_enveloppe', this.id);
        doc = (await this.user.db.get(this.id));
        user_timestamp = this.user.time(doc.timestamp);
        return this.ctx.prompt.play(`phrase:'message received:${index + 1}:${doc.caller_id}:${user_timestamp}'`);
      }

      // Create a new voicemail record in the database
      // ---------------------------------------------
      create() {
        var id_timestamp, msg;
        id_timestamp = timestamp();
        this.id = 'voicemail:' + id_timestamp + this.ctx.data.variable_uuid;
        debug('create', this.id);
        msg = {
          type: "voicemail",
          _id: this.id,
          timestamp: id_timestamp,
          box: 'new', // In which box is this message?
          caller_id: this.ctx.source,
          recipient: this.ctx.destination
        };
        // If the user simply hungs up this is the only event we will receive.
        // Note: now that we process `linger` properly this might be moved into `post_recording`, but the added complexity is probably not worth it.
        this.ctx.call.once('cleanup_linger', foot(async() => {
          debug('Disconnect Notice', this.id);
          this.done = true;
          await sleep(15000);
          heal(this.notify('create'));
        }));
        // Create new CDB record to hold the voicemail metadata
        return this.user.db.put(msg).catch((e) => {
          debug.csr(`Could not create message: ${e}`, this.id);
          return this.ctx.prompt.error('MSG-180');
        });
      }

      async notify(flag) {
        var name, notifier, ref3;
        debug('notify', flag, this.user.id, this.id);
        if (this.ctx.cfg.notifiers == null) {
          return;
        }
        ref3 = this.ctx.cfg.notifiers;
        for (name in ref3) {
          notifier = ref3[name];
          await ((name, notifier) => {
            return notifier(this.user, this.id, flag).catch(function(error) {
              var ref4;
              return debug.csr(`Notifier ${name} error: ${(ref4 = error.stack) != null ? ref4 : error}`);
            });
          })(name, notifier);
        }
      }

      async remove() {
        var doc;
        debug('remove', this.id);
        doc = (await this.user.db.get(this.id));
        doc.box = 'trash';
        await this.user.db.put(doc);
        heal(this.notify('remove'));
        return this.ctx.action('phrase', 'voicemail_ack,deleted');
      }

      async save() {
        var doc;
        debug('save', this.id);
        doc = (await this.user.db.get(this.id));
        doc.box = 'saved';
        await this.user.db.put(doc);
        heal(this.notify('save'));
        return this.ctx.action('phrase', 'voicemail_ack,saved');
      }

      // The forward operation is a bit complex since it requires to:

      // - locate and open the database for the destination user (I guess this can be done with `locate_user`);
      // - copy the JSON document from the original db to the destination db;
      // - copy the attachment(s);
      // - optionally add a new attachment (comment) in the destination (but not the source) db.

      // There used to be code properly handling "more than one attachment" in this module; however some of it was removed for ccnq4. Make sure all places know how to handle multi-part voicemails.
      async forward(destination) {
        var doc, gather_parts, i, j, len, messaging, name, new_doc, part, parts, sorted_attachments, target, user;
        messaging = new Messaging(this.ctx);
        ({user} = (await messaging.retrieve_number(destination)));
        if (user == null) {
          //# Blabla destination does not exist, try again
          return false;
        }
        this.ctx.source = this.user.number;
        this.ctx.destination = user.number;
        target = new Message(this.ctx, user);
        await target.create();
        // Record an additional part, which should be put as the first part (and the remaining parts should be shifted).
        await target.user.play_prompt();
        await target.start_recording();
        await target.post_recording();
        // So, mostly, we're left with:

        // Granted, downloading all the attachment in memory is not a good idea.
        doc = (await this.user.db.get(this.id, {
          attachments: true,
          binary: true
        }));
        new_doc = (await target.user.db.get(target.id, {
          attachments: true,
          binary: true
        }));
        // Append the parts of `doc` to the ones in `new_doc`.
        sorted_attachments = function(d) {
          return Object.keys(d._attachments).sort();
        };
        parts = [];
        gather_parts = function(d) {
          var j, len, m, name, ref3, results;
          ref3 = sorted_attachments(d);
          results = [];
          for (j = 0, len = ref3.length; j < len; j++) {
            name = ref3[j];
            if (m = name.match(/^part\d+\.(\S+)$/)) {
              results.push(parts.push({
                extension: m[1],
                value: d._attachments[name],
                duration: d.durations[name]
              }));
            }
          }
          return results;
        };
        // We need to rename the parts in `doc` so that they follow the ones in new_doc.
        gather_parts(new_doc);
        gather_parts(doc);
        new_doc._attachments = {};
        new_doc.durations = {};
        for (i = j = 0, len = parts.length; j < len; i = ++j) {
          part = parts[i];
          name = `part${i + target.the_first_part}.${part.extension}`;
          new_doc._attachments[name] = part.value;
          new_doc.durations[name] = part.duration;
        }
        new_doc.duration = sum_of(new_doc.durations);
        await target.user.db.put(new_doc);
        // Do not leak.
        await target.user.close_db();
        target.user = null;
        target = null;
        return true;
      }

    };

    Message.prototype.min_duration = parseInt((ref = process.env.MESSAGE_MIN_DURATION) != null ? ref : 2);

    Message.prototype.max_duration = parseInt((ref1 = process.env.MESSAGE_MAX_DURATION) != null ? ref1 : 300);

    Message.prototype.the_first_part = 1;

    Message.prototype.the_last_part = parseInt((ref2 = process.env.MAX_PARTS) != null ? ref2 : 9);

    return Message;

  }).call(this);

  module.exports = Message;

  pkg = require('../package.json');

  ({debug, foot, heal} = (require('tangible'))(`${pkg.name}:Message`));

  sleep = function(timeout) {
    return new Promise(function(resolve) {
      return setTimeout(resolve, timeout);
    });
  };

  Messaging = require('./Messaging');

  timestamp = function() {
    return new Date().toJSON();
  };

}).call(this);
