// Generated by CoffeeScript 2.4.1
(function() {
  var CouchDB, Messaging, Nimble, User, assert, debug, pkg;

  Messaging = class Messaging {
    constructor(ctx) {
      this.ctx = ctx;
      this.prov = new CouchDB((Nimble(this.ctx.cfg)).provisioning);
    }

    // Gather a customer phone number and locate that record.
    async gather_user(attempts = 3) {
      var number;
      debug('gather_user', {attempts});
      if (attempts <= 0) {
        return this.ctx.prompt.error();
      }
      number = (await this.ctx.prompt.get_number());
      if (number != null) {
        return this.locate_user(number, attempts - 1);
      } else {
        return this.gather_user(attempts - 1);
      }
    }

    async locate_user(number, attempts = 3) {
      var base, base1, base2, number_data, number_domain, ref, ref1, ref2, user, user_database;
      assert(number != null, 'locate_user: number must be defined');
      debug('locate_user', number, attempts);
      if (number === 'user-database') {
        return (await this._user(this.ctx.session.voicemail_user_id, this.ctx.session.voicemail_user_database));
      }
      // Locate endpoint-data (`User` will also need it, so store it in the session).
      // In most cases `session.endpoint` is already provided.

      // * session.endpoint_name Name of the endpoint used for locating the number-domain, set from hdr.X-En
      if ((base = this.ctx.session).endpoint_name == null) {
        base.endpoint_name = this.ctx.req.header('X-En');
      }
      // * session.endpoint Data of the endpoint used for locating the number-domain.
      if (this.ctx.session.endpoint_name != null) {
        if ((base1 = this.ctx.session).endpoint == null) {
          base1.endpoint = (await this.prov.get(`endpoint:${this.ctx.session.endpoint_name}`).catch((error) => {
            var ref;
            debug(`Endpoint ${this.ctx.session.endpoint_name} not found, ${(ref = error.stack) != null ? ref : error}.`);
            return null;
          }).then(function(data) {
            if (data != null ? data.disabled : void 0) {
              return {};
            } else {
              return data;
            }
          }));
        }
      } else {
        this.ctx.session.endpoint = null;
      }
      // Internal consistency
      if ((this.ctx.session.endpoint_name != null) && (this.ctx.session.endpoint == null)) {
        debug.dev(`Missing session.endpoint for ${this.ctx.session.endpoint_name}`);
        return this.ctx.prompt.error('MSI-48');
      }
      // Number-domain selection
      // -----------------------

      // In most cases we will already have a proper `number_domain` selected in the session.
      number_domain = (ref = this.ctx.session.number_domain) != null ? ref : null;
      // Fallback to the number-domain associated with the endpoint, if any.
      if (number_domain == null) {
        number_domain = (ref1 = this.ctx.session.endpoint) != null ? ref1.number_domain : void 0;
      }
      // Fallback to the default one configured.

      // * cfg.voicemail.number_domain The default number-domain used for voicemail, if none is found in session.number_domain nor session.endpoint.number_domain
      if (number_domain == null) {
        number_domain = (ref2 = this.ctx.cfg.voicemail.number_domain) != null ? ref2 : 'local';
        debug.csr(`No user_domain specified for ${number}, using configured ${number_domain} instead.`);
      }
      if ((base2 = this.ctx.session).number_domain == null) {
        base2.number_domain = number_domain;
      }
      ({number_data, user} = (await this.retrieve_number(number)));
      // A missing `user` might be due to the user mistyping their number when using `gather_user`, so give them another opportunity to do so.
      if (user == null) {
        return this.gather_user(attempts);
      }
      // Internal consistency
      assert(number_data != null, `Missing local number for ${user.id}`);
      ({user_database} = number_data);
      // If the record was found but no user-database is specified, either the line has no voicemail, or the record is incorrect. Either way, we can't proceed any further.
      if (user_database == null) {
        debug.csr(`Customer ${user.id} has no user_database.`);
        return this.ctx.prompt.error('MSI-42');
      }
      this.ctx.session.number = number_data;
      return user;
    }

    // Retrieve user based on number and optional user-data
    // ----------------------------------------------------
    async retrieve_number(number) {
      var new_number, number_data, number_domain, user, user_database, user_id;
      number_domain = this.ctx.session.number_domain;
      // Pass through any translation that the application may suggest.
      if (this.ctx.translate_local_number != null) {
        new_number = (await this.ctx.translate_local_number(number, number_domain));
        if (new_number != null) {
          number = new_number;
        }
      }
      // Attempt to locate the local-number record.
      user_id = `${number}@${number_domain}`;
      debug('retrieve_number >', user_id);
      // * session.number Data record of the local number for which we are handling voicemail.
      // * doc.local_number.user_database Name of the user's database.
      // * session.number.user_database Name of the user's database, see doc.local_number.user_database
      number_data = (await this.prov.get(`number:${user_id}`).catch(function(error) {
        debug.dev(`number:${user_id} not found, ${error}`);
        return {};
      }).then(function(data) {
        if (data != null ? data.disabled : void 0) {
          return {};
        } else {
          return data;
        }
      }));
      // Internal consistency
      if ((number_data != null ? number_data._id : void 0) == null) {
        debug(`No local number for ${user_id}`);
        return {};
      }
      // Use data from local-number
      ({user_database} = number_data);
      if (user_database == null) {
        debug(`Missing database for ${user_id}`);
        return {number_data};
      }
      user = (await this._user(user_id, user_database));
      debug('retrieve_number OK');
      return {number_data, user};
    }

    // Now that we have a user/local-number document, let's locate the associated user database.
    // Note: `userdb_base_uri` must contain authentication elements (e.g. "voicemail" user+pass)
    async _user(user_id, user_database) {
      var db_uri, user;
      // * cfg.userdb_base_uri The base URI concatenated with a doc.local_number.user_database name to access the user's database. It must contain any required authentication elements.
      db_uri = this.ctx.cfg.userdb_base_uri + '/' + user_database;
      user = new User(this.ctx, user_id, user_database, db_uri);
      await user.init_db();
      return user;
    }

  };

  module.exports = Messaging;

  pkg = require('../package.json');

  debug = (require('tangible'))(`${pkg.name}:Messaging`);

  assert = require('assert');

  User = require('./User');

  CouchDB = require('most-couchdb');

  Nimble = require('nimble-direction');

}).call(this);
