// Generated by CoffeeScript 2.4.1
(function() {
  var CouchDB, Formats, Message, URL, User, assert, couchapp, debug, ec, moment, pkg;

  Formats = require('./Formats');

  User = (function() {
    var ref, ref1;

    // User
    // ====
    class User {
      constructor(ctx, id, database, db_uri) {
        this.ctx = ctx;
        this.id = id;
        this.database = database;
        this.db_uri = db_uri;
        [this.number, this.number_domain] = this.id.split('@');
        this.db = new CouchDB(this.db_uri, true);
      }

      // Inject the views into the database.
      // Note: this requires the application to be database admin, which is OK.
      async init_db() {
        debug('init_db');
        await this.db.update(couchapp);
      }

      async close_db() {
        var base;
        await (typeof (base = this.db).close === "function" ? base.close() : void 0);
        return this.db = null;
      }

      uri(name, rev) {
        return this.ctx.voicemail_uri(this, 'voicemail_settings', name, rev);
      }

      voicemail_settings(no_error = false) {
        debug('voicemail_settings');
        // Memoize
        if (this.vm_settings != null) {
          return Promise.resolve(this.vm_settings);
        }
        return this.db.get('voicemail_settings').catch((error) => {
          // Debug as much as we can.
          debug.csr("VM Box is not available", {
            user_id: this.id
          });
          // Tell the user to call support.
          if (no_error) {
            return {};
          }
          return this.ctx.prompt.error('USR-41');
        }).then((doc) => {
          return this.vm_settings = doc; // Memoize
        });
      }

      
      // Convert a timestamp (ISO string) to a local timestamp (ISO string)

      // * doc.voicemail_settings.timezone Timezone for voicemail.
      timezone() {
        var ref2;
        return (ref2 = this.vm_settings.timezone) != null ? ref2 : this.default_timezone;
      }

      time(t) {
        var timezone;
        debug('time');
        timezone = this.timezone();
        if (timezone != null) {
          return moment.tz(t, timezone).format();
        } else {
          return moment(t).format();
        }
      }

      // Playing prompt before recording a voice message
      // -----------------------------------------------
      async play_prompt() {
        var f, find, vm_settings;
        debug('play_prompt');
        vm_settings = (await this.voicemail_settings());
        find = function(name) {
          return Formats.find(vm_settings, name);
        };
        switch (false) {
          // ### Selecting a prompt

          // The prompt to be played may be one of:
          // - a recorded message;
          // - an announceiment with the recorded full name of the recipient;
          // - or a generic announcement.

          // The user might indicate which announcement they'd like to play; otherwise an announcement is automatically selected based on the available recordings.
          case !(vm_settings.prompt === 'prompt' && (f = find('prompt'))):
            await this.ctx.prompt.play(this.uri(f));
            break;
          case !(vm_settings.prompt === 'name' && (f = find('name'))):
            await this.ctx.prompt.play(this.uri(f));
            await this.ctx.prompt.phrase('voicemail_unavailable');
            break;
          case vm_settings.prompt !== 'default':
            await this.ctx.prompt.phrase(`voicemail_play_greeting,${this.id}`);
            break;
          case !(f = find('prompt')):
            await this.ctx.prompt.play(this.uri(f));
            break;
          case !(f = find('name')):
            await this.ctx.prompt.play(this.uri(f));
            await this.ctx.prompt.phrase('voicemail_unavailable');
            break;
          default:
            await this.ctx.prompt.phrase(`voicemail_play_greeting,${this.id}`);
        }
        // * doc.voicemail_settings.prompt (optional string, either 'prompt', 'name', or 'default') Indicate how the user would like the caller to be prompted to leave a message. If not present, the choice is made based on which attachment is present.
        // * doc.voicemail_settings._attachments.prompt (prompt.wav) User-specified voicemail prompt. Used if present.
        // * doc.voicemail_settings._attachments.name (name.wav) User-specified voicemail name. Used if the prompt is not present.

        // ### Actually recording the message

        // The user might opt for an announcement-only voicemailbox, in which case the caller does not have the option to leave a message.

        // * doc.voicemail_settings.do_not_record If true, do not record voicemail messages.
        if (this.vm_settings.do_not_record) {
          return false;
        }
        await this.ctx.action('phrase', 'voicemail_record_message');
        return true;
      }

      async authenticate(attempts) {
        var authenticated, pin, ref2, vm_settings;
        debug('authenticate', {attempts});
        // As long as we went through `locate_user` these should be provided.
        assert(this.ctx.session.number != null, 'Missing session number data');
        if (attempts == null) {
          attempts = 3;
        }
        if (attempts <= 0) {
          return this.ctx.prompt.error();
        }
        // * doc.voicemail_settings Document found in the user database. Contains parameters for that user's voicemail box.
        vm_settings = (await this.voicemail_settings());
        authenticated = false;
        // If the user requested not to be queried for a PIN, we authenticate using the endpoint.

        // * doc.voicemail_settings.ask_pin If false, bypass authentication by PIN and validate access using the endpoint.
        if (vm_settings.ask_pin === false) {
          // ... however let's make sure we don't compare `null` with `null`, which would lead us to allow entry to non-validated calls.
          if (((ref2 = this.ctx.session.endpoint) != null ? ref2.endpoint : void 0) != null) {
            authenticated = this.ctx.session.number.endpoint === this.ctx.session.endpoint.endpoint;
          }
        }
        // Otherwise, authentication can only happen with the PIN.

        // * doc.voicemail_settings.pin The (numeric) PIN to access voicemail.
        if (!authenticated) {
          if (vm_settings.pin != null) {
            pin = (await this.ctx.prompt.get_pin());
            authenticated = pin === vm_settings.pin;
          }
        }
        // * doc.voicemail_settings.language Language used inside voicemail.
        if (authenticated) {
          if (vm_settings.language != null) {
            await this.ctx.set({
              language: vm_settings.language
            });
          }
          return (await this.ctx.prompt.phrase('voicemail_hello'));
        } else {
          return this.authenticate(attempts - 1);
        }
      }

      async get_new_messages() {
        var rows;
        debug('get_new_messages');
        rows = [];
        await this.db.query('voicemail', 'new_messages').observe(function(row) {
          return rows.push(row);
        });
        return rows;
      }

      async new_messages() {
        var rows;
        debug('new_messages');
        rows = (await this.get_new_messages());
        await this.ctx.action('phrase', `voicemail_message_count,${rows.length}:new`);
        return rows;
      }

      async get_saved_messages() {
        var rows;
        debug('get_saved_messages');
        rows = [];
        await this.db.query('voicemail', 'saved_messages').observe(function(row) {
          return rows.push(row);
        });
        return rows;
      }

      async saved_messages() {
        var rows;
        rows = (await this.get_saved_messages());
        await this.ctx.action('phrase', `voicemail_message_count,${rows.length}:saved`);
        return rows;
      }

      async navigate_messages(rows, current) {
        var choice, destination, msg;
        debug('navigate_messages', {
          rows: rows != null ? rows.length : void 0,
          current
        });
        // Exit once we reach the end or there are no messages, etc.
        if (current < 0 || (rows == null) || current >= rows.length) {
          return;
        }
        msg = new Message(this.ctx, this, rows[current].id);
        choice = (await msg.play_enveloppe(current));
        if (choice == null) {
          choice = (await msg.play_recording());
        }
        if (choice == null) {
          choice = (await this.ctx.prompt.get_choice("phrase:'voicemail_listen_file_check:1:2:3:4'"));
        }
        switch (choice) {
          // Listen to the current message
          case '1':
            return this.navigate_messages(rows, current);
          // Jump to the previous message
          case "7":
            if (current === 0) {
              await this.ctx.prompt.phrase('no previous message');
              return this.navigate_messages(rows, current);
            } else {
              return this.navigate_messages(rows, current - 1);
            }
            break;
          // Jump to the next message
          case "9":
            if (current === rows.length - 1) {
              await this.ctx.prompt.phrase('no next message');
              return this.navigate_messages(rows, current);
            } else {
              return this.navigate_messages(rows, current + 1);
            }
            break;
          // Delete
          case "3":
            await msg.remove();
            return this.navigate_messages(rows, current + 1);
          // Save
          case "2":
            await msg.save();
            return this.navigate_messages(rows, current + 1);
          // Forward
          case "4":
            // Gather recipient's number
            destination = (await this.ctx.prompt.get_number({
              file: 'phrase:voicemail_forward_message_enter_extension:#',
              invalid_file: 'phrase:voicemail_invalid_extension'
            }));
            // Attempt to forward
            if (destination != null) {
              if (!(await msg.forward(destination))) {
                await this.ctx.prompt.phrase('voicemail_invalid_extension');
              }
            }
            // Repeat message so that the user knows where to continue
            return this.navigate_messages(rows, current);
          // Return to the main menu
          case "0":
            return true;
          default:
            // Default navigation is: read next message or return to the main menu
            if (current === rows.length - 1) {
              return;
            }
            return this.navigate_messages(rows, current + 1);
        }
      }

      async config_menu(attempt = 3) {
        var choice;
        debug('config_menu');
        if ((this.ctx.call == null) || this.ctx.call.closed) {
          return;
        }
        choice = (await this.ctx.prompt.get_choice("phrase:'voicemail_config_menu:1:2:3:4:5'"));
        switch (choice) {
          case "1":
            await this.record_greeting();
            return this.config_menu();
          case "3":
            await this.record_name();
            return this.config_menu();
          case "4":
            await this.change_password();
            return this.config_menu();
          case "5":
            return this.main_menu();
          default:
            if (attempt > 0) {
              return this.config_menu(attempt - 1);
            } else {
              return this.main_menu();
            }
        }
      }

      async main_menu(attempt = 7) {
        var choice, rows;
        debug('main_menu');
        if ((this.ctx.call == null) || this.ctx.call.closed) {
          return;
        }
        choice = (await this.ctx.prompt.get_choice("phrase:'voicemail_menu:1:2:3:4'"));
        debug('main_menu', {choice});
        switch (choice) {
          case "1":
            rows = (await this.new_messages());
            await this.navigate_messages(rows, 0);
            return this.main_menu();
          case "2":
            rows = (await this.saved_messages());
            await this.navigate_messages(rows, 0);
            return this.main_menu();
          case "3":
            return this.config_menu();
          case "4":
            return this.ctx.prompt.goodbye();
          default:
            if (attempt > 0) {
              return this.main_menu(attempt - 1);
            } else {
              return this.ctx.prompt.goodbye();
            }
        }
      }

      // * doc.voicemail_settings._attachments Contains prompts for the user's voicemail.
      async record_something(that, phrase, min_duration = 3) {
        var doc, name, recorded, rev, upload_url;
        debug('record_something', {that, phrase});
        doc = (await this.db.get('voicemail_settings'));
        rev = doc._rev;
        await this.ctx.prompt.phrase(phrase);
        name = Formats.name(that);
        upload_url = this.uri(name, rev);
        recorded = (await this.ctx.prompt.record(upload_url));
        if (recorded < min_duration) {
          await this.ctx.prompt.phrase('vm_say,too short');
          this.record_something(that, phrase);
        } else {
          await this.ctx.prompt.phrase('vm_say,thank you');
        }
      }

      record_greeting() {
        debug('record_greeting');
        return this.record_something('prompt', 'voicemail_record_greeting').catch((error) => {
          debug(`record_greeting: ${error}`);
          return this.ctx.prompt.error('USR-263');
        });
      }

      record_name() {
        debug('record_name');
        return this.record_something('name', 'voicemail_record_name', 1).catch((error) => {
          debug(`record_name: ${error}`);
          return this.ctx.prompt.error('USR-270');
        });
      }

      async change_password() {
        var get_pin, new_pin, vm_settings;
        debug('change_password');
        if ((this.ctx.call == null) || this.ctx.call.closed) {
          return;
        }
        get_pin = async() => {
          var pin;
          pin = (await this.ctx.prompt.get_new_pin({
            min: this.min_pin_length
          }));
          if (pin != null) {
            if (pin.length >= this.min_pin_length) {
              return pin;
            } else {
              await this.ctx.prompt.phrase('vm_say,too short');
            }
          }
          return get_pin();
        };
        new_pin = (await get_pin());
        vm_settings = (await this.db.get('voicemail_settings'));
        vm_settings.pin = new_pin;
        await this.db.put(vm_settings);
        delete this.vm_settings; // remove memoized value
        await this.ctx.prompt.phrase('vm_say,thank you');
      }

    };

    User.prototype.min_pin_length = parseInt((ref = process.env.MIN_PIN_LENGTH) != null ? ref : 6);

    User.prototype.default_timezone = (ref1 = process.env.DEFAULT_TIMEZONE) != null ? ref1 : null;

    User.prototype.voicemail_dir = '/opt/freeswitch/messages';

    return User;

  }).call(this);

  module.exports = User;

  pkg = require('../package.json');

  debug = (require('tangible'))(`${pkg.name}:User`);

  assert = require('assert');

  moment = require('moment-timezone');

  CouchDB = require('most-couchdb/with-update');

  ec = encodeURIComponent;

  ({URL} = require('url'));

  Message = require('./Message');

  couchapp = require('./couchapp');

}).call(this);
